﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Info2Praktikum;

namespace _2011_05_04_Praktikum
{
    public partial class MainWindow : Window
    {
        Microsoft.Win32.OpenFileDialog dialogOpen = new Microsoft.Win32.OpenFileDialog();
        Microsoft.Win32.SaveFileDialog dialogSave = new Microsoft.Win32.SaveFileDialog();
        Bitmap inputBitmap, outputBitmap;

        public MainWindow()
        {
            InitializeComponent();

            dialogOpen.Filter = "JPEG-Dateien|*.jpg|PNG-Dateien|*.png|BMP-Dateien|*.png|mein Dateityp|*.textbitmap|alle Dateien|*.*";
            dialogSave.Filter = "mein Dateityp|*.textbitmap";
            dialogSave.AddExtension = true;

            listBoxFilter.Items.Add(new BrightnessAdjuster(this, 0.5));
            listBoxFilter.Items.Add(new Inverter());
            listBoxFilter.Items.Add(new HorizontalFlipper());
            listBoxFilter.Items.Add(new BrightnessAdjuster(this, 1.5));
            listBoxFilter.Items.Add(new Cropper(0, 0, 10, 10));
            listBoxFilter.Items.Add(new Cropper(200, 200, 100, 100));
        }

        public void RefreshItems()
        {
            listBoxFilter.Items.Refresh();
        }

        private void buttonLoad_Click(object sender, RoutedEventArgs e)
        {
            bool? result = dialogOpen.ShowDialog();
            if (result.HasValue && result.Value)
            {
                System.IO.FileInfo fi = new System.IO.FileInfo(dialogOpen.FileName);
                if (fi.Extension == ".textbitmap")
                {
                    inputBitmap = Bitmap.ReadFromText(dialogOpen.FileName);
                }
                else
                {
                    inputBitmap = new Bitmap(dialogOpen.FileName);
                }

                image.Source = inputBitmap.GetImage();
            }
        }

        private void buttonCompute_Click(object sender, RoutedEventArgs e)
        {
            Bitmap b = inputBitmap;

            // einfache Version: Reihenfolge wie angeklickt, nicht wie in der Liste
            // for (int i = 0; i < listBoxFilter.SelectedItems.Count; i++)
            // {
            //     b = ((Filter)listBoxFilter.SelectedItems[i]).Execute(b);
            // }

            // aufwändigere Version: Reihenfolge wie in der Liste
            for (int i = 0; i < listBoxFilter.Items.Count; i++)
            {
                Filter f = (Filter)listBoxFilter.Items[i];
                if (listBoxFilter.SelectedItems.Contains(f))
                {
                    b = f.Execute(b);
                }
            }

            outputBitmap = b;
            image.Source = outputBitmap.GetImage();
        }

        private void buttonSave_Click(object sender, RoutedEventArgs e)
        {
            if (outputBitmap != null)
            {
                bool? result = dialogSave.ShowDialog();
                if (result.HasValue && result.Value)
                {
                    outputBitmap.WriteAsText(dialogSave.FileName);
                }
            }
        }

        private void buttonSettings_Click(object sender, RoutedEventArgs e)
        {
            foreach (Filter f in listBoxFilter.Items)
            {
                f.ShowSettings();
            }
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            foreach (Filter f in listBoxFilter.Items)
            {
                f.CloseSettings();
            }
        }
    }
}
